# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Audit log context resolvers."""

from flask import current_app, request
from invenio_records.dictutils import dict_lookup, dict_set
from invenio_users_resources.entity_resolvers import UserResolver


class UserContext(object):
    """Payload generator for audit log using the user entity resolver."""

    def __init__(self, key="user"):
        """Ctor."""
        self.key = key
        self.resolver = UserResolver()

    def __call__(self, data, lookup_key="user_id", **kwargs):
        """Update data with resolved user data."""
        entity_ref = dict_lookup(data, lookup_key)
        entity_proxy = self.resolver.get_entity_proxy({self.key: entity_ref})
        entity_data = entity_proxy.resolve()
        dict_set(data, self.key, entity_data)


class RecordContext(object):
    """Payload generator for audit log to get record auditing metadata."""

    def __call__(self, data, **kwargs):
        """Update data with resolved record data."""
        record = kwargs["record"]
        if current_app.config["AUDIT_LOGS_METADATA_FIELDS"]["revision_id"]:
            record_versions = record.model.versions.all()
            dict_set(data, "metadata.revision_id", record_versions[-1].transaction_id)
        if current_app.config["AUDIT_LOGS_METADATA_FIELDS"]["parent_pid"]:
            dict_set(data, "metadata.parent_pid", record.parent.pid.pid_value)


class RequestContext(object):
    """Payload generator for audit log using the request context."""

    def __call__(self, data, **kwargs):
        """Update data with resolved request data."""
        # IMPORTANT! DON'T COPY THIS, PLEASE DON'T DO THIS EVER...
        if request:
            if current_app.config["AUDIT_LOGS_METADATA_FIELDS"]["ip_address"]:
                ip = request.headers.get("REMOTE_ADDR") or request.remote_addr
                if ip:
                    dict_set(data, "metadata.ip_address", ip)

            if current_app.config["AUDIT_LOGS_METADATA_FIELDS"]["session"]:
                session = request.cookies.get("SESSION") or request.cookies.get(
                    "session"
                )
                if session:
                    dict_set(data, "metadata.session", session)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2025 CERN.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Action registration via entrypoint function."""

from invenio_audit_logs.services import AuditLogAction

from .context import (
    RecordContext,
    RequestContext,
    UserContext,
)


class RecordBaseAuditLog(AuditLogAction):
    """Base class for audit log builders."""

    context = [
        UserContext(),
        RequestContext(),
    ]
    resource_type = "record"


class DraftCreateAuditLog(RecordBaseAuditLog):
    """Audit log for draft creation."""

    id = "draft.create"
    message_template = "User {user_id} created the draft {resource_id}."


class DraftEditAuditLog(RecordBaseAuditLog):
    """Audit log for draft editing."""

    id = "draft.edit"
    message_template = "User {user_id} updated the draft {resource_id}."


class RecordPublishAuditLog(RecordBaseAuditLog):
    """Audit log for record publication."""

    context = RecordBaseAuditLog.context + [
        RecordContext(),
    ]

    id = "record.publish"
    message_template = "User {user_id} published the record {resource_id}."


class DraftDeleteAuditLog(RecordBaseAuditLog):
    """Audit log for draft deletion."""

    id = "draft.delete"
    message_template = "User {user_id} deleted the draft {resource_id}."


class DraftNewVersionAuditLog(RecordBaseAuditLog):
    """Audit log for new draft version creation."""

    id = "draft.new_version"
    message_template = "User {user_id} created a new version {resource_id}."



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2022 CERN.
# Copyright (C) 2020      Northwestern University.
# Copyright (C) 2021      TU Wien.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Invenio Drafts Resources module to create REST APIs"""

from setuptools import setup

setup()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2020 CERN.
# Copyright (C) 2025 Graz University of Technology.
#
# Invenio-Drafts-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Draft Models API."""

from datetime import datetime, timezone

from invenio_db import db
from invenio_records.models import RecordMetadataBase
from sqlalchemy.dialects import mysql
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy_utils.types import UUIDType


class ParentRecordMixin:
    """A mixin factory that add the foreign keys to the parent record.

    Usage:

    .. code-block:: python

        class MyRecord(db.Model, RecordMetadataBase, ParentRecordMixin):
            __parent_record_model__ = ParentRecordMetadata
    """

    __parent_record_model__ = None

    @declared_attr
    def parent_id(cls):
        """Parent identifier."""
        # We restrict deletion of the parent record in case a record or draft
        # exists via database-level trigger.
        return db.Column(
            UUIDType, db.ForeignKey(cls.__parent_record_model__.id, ondelete="RESTRICT")
        )

    @declared_attr
    def parent(cls):
        """Relationship to parent record."""
        return db.relationship(cls.__parent_record_model__)

    index = db.Column(db.Integer, nullable=True)
    """The version index of the record."""


class ParentRecordStateMixin:
    """Database model mixin to keep the state of the latest and next version.

    We keep this data outside the parent record itself, because we want to
    update it without impacting the parent record's version counter. The
    version counter in the parent record we use to determine if we have to
    reindex all record versions.

    Usage:

    .. code-block:: python

        class MyParentState(db.Model, ParentRecordState):
            __parent_record_model__ = MyParentRecord
            __draft_model__ = MyDraft
            __record_model__ = MyRecord
    """

    __parent_record_model__ = None
    __record_model__ = None
    __draft_model__ = None

    @declared_attr
    def parent_id(cls):
        """Parent record identifier."""
        return db.Column(
            UUIDType,
            # 1) If the parent record is deleted, we automatically delete
            # the state as well via database-level on delete trigger.
            # 2) A parent can only be deleted, if it has no drafts/records via
            # FKs from drafts/records to the parent.
            db.ForeignKey(cls.__parent_record_model__.id, ondelete="CASCADE"),
            primary_key=True,
        )

    @declared_attr
    def latest_id(cls):
        """UUID of the latest published record/draft.

        Note, since a record and draft share the same UUID, the UUID can be
        used to get both the record or the draft. It's a foreign key to the
        record to ensure that the record exists (and thus is published).

        If no record has been published, the value is None.
        """
        return db.Column(
            UUIDType,
            db.ForeignKey(cls.__record_model__.id),
            nullable=True,
        )

    latest_index = db.Column(db.Integer, nullable=True)
    """The index of the latest published record.

    If no record has been published, the value is None.
    """

    @declared_attr
    def next_draft_id(cls):
        """UUID of the draft for the next version (yet to be published).

        Note, since a record and draft share the same UUID, the UUID can be
        used to get both the record or the draft. It's a foreign key to the
        draft, because we use it to track if a draft has been created for the
        next version.

        If no draft for the next version has been created, the value is None.
        """
        return db.Column(
            UUIDType,
            db.ForeignKey(cls.__draft_model__.id),
            nullable=True,
        )


class DraftMetadataBase(RecordMetadataBase):
    """Represent a base class for draft metadata."""

    fork_version_id = db.Column(db.Integer)
    """Version ID of the record."""

    expires_at = db.Column(
        db.UTCDateTime(),
        default=lambda: datetime.now(timezone.utc),
        nullable=True,
    )
    """Specifies when the draft expires. If `NULL` the draft doesn't expire."""
